pub struct StyledStringBuilder { /* private fields */ }Expand description
Builder for StyledStrings.
A builder can be initialized from scratch via StyledString::builder() or Default.
Alternatively, it can be initialized from an existing StyledString.
A builder can be extended by pushing text, styles
or StyledStrings into it.
Examples
let mut builder = StyledString::builder();
builder.push_style(AnsiColor::BrightGreen.on(AnsiColor::White).bold());
builder.push_text("Hello");
builder.push_text(",");
// It's possible to use `+=` as syntactic sugar for `push_str()` / `push_style()`.
builder += Style::new();
builder.push_text(" world");
builder += styled!("[[it, dim]]!");
let s = builder.build();
assert_eq!(
s.to_string(),
"[[bold green! on white]]Hello,[[/]] world[[italic dim]]!"
);Implementations
Sourceimpl StyledStringBuilder
impl StyledStringBuilder
Sourcepub fn push_text(&mut self, text: &str)
pub fn push_text(&mut self, text: &str)
Pushes unstyled text at the end of the string.
Panics
Panics if text contains an ANSI escape char (\x1b).
Sourcepub fn push_style(&mut self, style: Style)
pub fn push_style(&mut self, style: Style)
Pushes a style at the end of this string.
Sourcepub fn build(self) -> StyledString
pub fn build(self) -> StyledString
Finalizes the StyledString.